/**
 * author: Matthias Bernt, Tom Hartmann
 * coauthors: M. Middendorf
 */

#include <algorithm>
#include <iostream>
#include <iterator>
#include <vector>
#include <sstream>
#include <limits>

#include "/opt/gurobi810/linux64/include/gurobi_c++.h"
#include "ilp-rrrmt.hpp"

using namespace std;

// helps to understand what is going on in the ILP
void printmodel(const GRBModel &model, int maxDis, float currentBest, int upperBound,
		double last_scenario_cost, unsigned int numb_scen_sol, float cutoff);

//#define ILP_DEBUG
//#define ILP_ALL_DEBUG


// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// construct an emptyscan scenario
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

ilp_rrrmt::ilp_rrrmt() : ordered(){}

/*ilp_rrrmt::ilp_rrrmt( const genom &g1, const genom &g2, const costfoo *cst, float &objV, bool output){
	genom cpg1 = g1;
	//	get size of the genomes
	int genomSize;
	genomSize = g1.size()+1;

	// test: do both genomes hae the same size
	if(g2.size()!=genomSize-1){
		cout << "Both genomes do not have the same size!" << endl;
	}

	// identify_g returns the names of the genes of the renamed genome
	genom gpi; 				// contains the names of the renamed genomes

//	cout << "ILP g1: " << g1 << endl;
//	cout << "ILP g2: " << g2 << endl;
	gpi = g2.identify_g(g1);
//	cout << "ILP g1 after in.: " << g1 << endl;
//	cout << "ILP g2 after in.: " << g2 << endl;
//
//	cout << endl;


	// get costs of rrrmt events

	rrrmt *i;
	i = new rev( 1, 2, gpi );
	float costI = (*cst)[i];
	rrrmt *iT;
	iT = new revtransp( 1, 2, 3, gpi );
	float costIT = (*cst)[iT];
	rrrmt *T;
	T = new transp(1,2,3, gpi);
	float costT = (*cst)[T];

	// TODO
	// construct maximum distance
	//int maxDis;

	// contains O matrix of the start genome
	int** genomStart = new int*[genomSize];
	for(int i = 0; i < genomSize; i++){
	    genomStart[i] = new int[genomSize];
	}

	for(int i=0; i< genomSize; i++){
		for(int j=0; j < genomSize; j++){
			genomStart[i][j]=0;
		}
	}

	// temp variables
	int tempGPiI;
	int tempGPiJ;

	for(int i=0; i<genomSize-1; i++){
		for(int j=0; j<genomSize-1; j++){
			tempGPiI = abs( gpi[i] );
			tempGPiJ = abs( gpi[j] );
			if(i<j){
				genomStart[tempGPiI-1][tempGPiJ-1]=1;
			}else if(i>j){
				genomStart[tempGPiJ-1][tempGPiI-1]=1;
			}
		}
	}

	// sets genomStart for the dummy element
	for(int i=0; i<genomSize; i++){
		genomStart[genomSize-1][i]=0;
		if(i!=genomSize-1){
			genomStart[i][genomSize-1]=1;
		}
	}

///////////////	add ILP //////////////////////////////////////////////////////////////////////////////////////////////
	for(int maxDis = 2; maxDis<genomSize; maxDis++){
//		cout << "////////////////////////////"<< "distance ="<< maxDis-1 << "////////////////////////////////////////" << endl;

	try {
		// create gurobi environement
	    GRBEnv env = GRBEnv();
	    // produce output ??

	    if(!output){
			env.set(GRB_IntParam_OutputFlag,0);
		}
	    // only use max number of threads
	    env.set(GRB_IntParam_Threads, 0);
	    // create gurobi model
	    GRBModel model = GRBModel(env);

//////////////////////////////// Create variables ////////////////////////////////////////////////////////////////////////

	    // O_ijk stores the permutation
	    GRBVar*** O;
	    O = new GRBVar**[maxDis];
	    for( int k=0; k < maxDis; k++){
	    	O[k] = new GRBVar*[genomSize];
	    	for(int i=0; i< genomSize; i++){
	    		O[k][i] = new GRBVar[genomSize];
	    	}
	    }

	    for(int k=0; k<maxDis; k++){
	    	for(int i=0; i<genomSize; i++){
	    		for(int j=0; j< genomSize; j++){
	    			O[k][i][j] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
	    		}
	    	}
	    }

	    // S_ik contains the sign of a gene, S_ik = 1 <-> gene i has a negative sign
	    GRBVar** S;
	    S = new GRBVar*[maxDis];
	    for( int k=0; k< maxDis; k++){
	    	S[k]= new GRBVar[genomSize];
	    }

	    for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				S[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

	    // I_k = 1 <=> pi_k was changed by an inversion to pi_(k+1)
	    GRBVar* I;
		I = new GRBVar[maxDis];
	    for(int k=0; k<maxDis; k++){
				I[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
		}

	    // T_k = 1 <=> pi_k was changed by a transposition to pi_(k+1)
		GRBVar* T;
		T = new GRBVar[maxDis];
		for(int k=0; k<maxDis; k++){
				T[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
		}

		// U_k = 1 <=> pi_k was changed by an inverse transposition to pi_(k+1)
		GRBVar* U;
		U = new GRBVar[maxDis];
		for(int k=0; k<maxDis; k++){
				U[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
		}

	    // BL_ik left breakpoint variable
	    GRBVar** BL;
		BL = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			BL[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				BL[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// BM_ik middle breakpoint variable
		GRBVar** BM;
		BM = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			BM[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				BM[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// BR_ik right breakpoint variable
		GRBVar** BR;
		BR = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			BR[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				BR[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// CL_k = 1 <=> the interval between BL and BM gets inverted
		GRBVar* CL;
		CL = new GRBVar[maxDis];
		for(int k=0; k<maxDis; k++){
			CL[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
		}

		// CR_k = 1 <=> the interval between BM and BR gets inverted
		GRBVar* CR;
		CR = new GRBVar[maxDis];
		for(int k=0; k<maxDis; k++){
			CR[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
		}

		// WL_ek =1 <=> element e is within the left interval
		GRBVar** WL;
		WL = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			WL[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				WL[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// WR_ek =1 <=> element e is within the right interval
		GRBVar** WR;
		WR = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			WR[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				WR[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// LM_ek =1 <=> element e is to the left of the middle bonding element
		GRBVar** LM;
		LM = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			LM[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				LM[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// LR_ek =1 <=> element e is to the left of the right bonding element
		GRBVar** LR;
		LR = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			LR[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				LR[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// RL_ek =1 <=> element e is to the right of the left bonding element (including bounding element)
		GRBVar** RL;
		RL = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			RL[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				RL[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		// RM_ek =1 <=> element e is to the right of the middle bonding element (including bounding element)
		GRBVar** RM;
		RM = new GRBVar*[maxDis];
		for( int k=0; k< maxDis; k++){
			RM[k]= new GRBVar[genomSize];
		}

		for(int k=0; k<maxDis; k++){
			for(int i=0; i<genomSize; i++){
				RM[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
			}
		}

		model.update();

	    // Set objective:
	    GRBLinExpr obj = 0;
	    for(int k =0 ; k< maxDis-1; k++){
	    	obj += I[k]*costI + T[k]*costT + U[k]*costIT;
	    }

	    model.setObjective(obj, GRB_MINIMIZE);

///////////////////////////////////////Add constraints////////////////////////////////////////////////////////////

	    // Add initial constraints for the starting & target genome, (1)
	    for(int i=0; i< genomSize;i++){
	    	for(int j=0; j< genomSize; j++){
	    		// starting genome
	    		model.addConstr( O[0][i][j] - genomStart[i][j] <= 0 );
	    		model.addConstr( O[0][i][j] - genomStart[i][j] >= 0 );
	    		// target genome
	    		if(i>j){
	    			model.addConstr( O[maxDis-1][i][j] <= 0);
	    			model.addConstr( O[maxDis-1][i][j] >= 0);
	    		}
	    		if(i<j){
	    			model.addConstr( O[maxDis-1][i][j] -1 <= 0);
	    			model.addConstr( O[maxDis-1][i][j] -1 >= 0);
	    		}
	    	}
	    }

	    // Add initial constraints for the signs of starting & target genome (based on genes), (2)
	    for(int i=0; i<genomSize-1;i++){
	    	// starting genome signs
	    	if(gpi[i]<0){
	    		model.addConstr( S[0][ (gpi[i]*(-1)) -1] -1 <= 0);
	    		model.addConstr( S[0][ (gpi[i]*(-1)) -1] -1 >= 0);
	    	}
	    	else{
	    		model.addConstr( S[0][ gpi[i]-1 ] <= 0);
	    		model.addConstr( S[0][ gpi[i]-1 ] >= 0);
	    	}
	    	// target genome signs
	    	model.addConstr(S[maxDis-1][i] <= 0);
	    	model.addConstr(S[maxDis-1][i] >= 0);

	    }

	    // sets O_ijk variables for the dummy element
	    for(int k=0; k<maxDis; k++){
	    	for(int i=0; i<genomSize;i++){
	    		model.addConstr(O[k][genomSize-1][i]>=0);
	    		model.addConstr(O[k][genomSize-1][i]<=0);
				if(i!=genomSize-1){
					model.addConstr(O[k][i][genomSize-1]<=1);
					model.addConstr(O[k][i][genomSize-1]>=1);
				}
	    	}
	    }

	    //sets signs of dummy element
	    for(int k=0; k<maxDis; k++){
	    	model.addConstr(S[k][genomSize-1]>=0);
	    	model.addConstr(S[k][genomSize-1]<=0);
	    }

	   // Add constraints, that only one rearrangement at step k is allowed, (4)
	   for(int k=0; k<maxDis; k++){
		   model.addConstr(I[k]+T[k]+U[k] -1 <= 0);
		   model.addConstr(I[k]+T[k]+U[k] -1 >= 0);
	   }

	   // in every genome, there exists only one BL, BM, BR, (5)
	   GRBLinExpr tempBL = 0;
	   GRBLinExpr tempBM = 0;
	   GRBLinExpr tempBR = 0;
	   for(int k=0; k < maxDis; k++){
		   tempBL =0;
		   tempBM =0;
		   tempBR =0;
		   for(int i=0; i < genomSize; i++){
			   tempBL += BL[k][i];
			   tempBM += BM[k][i];
			   tempBR += BR[k][i];
		   }
		   model.addConstr(tempBL -1 <=0);
		   model.addConstr(tempBL -1 >=0);
		   model.addConstr(tempBM -1 <=0);
		   model.addConstr(tempBM -1 >=0);
		   model.addConstr(tempBR -1 <=0);
		   model.addConstr(tempBR -1 >=0);
	   }

	   // left bounding element has to be left of the middle one, (6)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int j=0; j < genomSize; j++){
					  model.addConstr(5*(1-BL[k][i])+ 5*(1-BM[k][j]) + O[k][i][j] -1 >=0);
			   }
		   }
	   }

	   // for transpositions the middle bound must be left of the right bound, (7)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int j=0; j < genomSize; j++){
					   model.addConstr(5*(1-T[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) + O[k][i][j] -1 >=0);
			   }
		   }
	   }

	   // for inverse transpositions the middle bound must be left of the right bound, (8)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int j=0; j < genomSize; j++){
					   model.addConstr(5*(1-U[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) + O[k][i][j] -1 >=0);
			   }
		   }
	   }

	   // for inversion the middle and the right bound are identical, (9)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int j=0; j < genomSize; j++){
				   model.addConstr(5*(1-I[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) + O[k][i][j] -O[k][j][i] >=0);
				   model.addConstr(5*(1-I[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) - O[k][i][j] +O[k][j][i] >=0);
			   }
		   }
	   }

	   // if I_k =1 => the left interval gets turned and the right interval not, thus CL_k =1 and CR_k=0, (11)
	   for(int k=0; k<maxDis; k++){
		   model.addConstr(I[k]-CL[k] <=0);
		   model.addConstr(1-I[k]-CR[k] >=0);
	   }

	   // if T_k=1 => no interval gets inverted, thus CL_k=CR_k=0, (12)
	   for(int k=0; k<maxDis; k++){
		   model.addConstr(5*(1-T[k])-CL[k]-CR[k] >=0 );
	   }

	   // if U_k=1 => exactly one of both intervals gets inverted, (13)
	   for(int k=0; k<maxDis; k++){
		   model.addConstr(5*(1-U[k])+1-CL[k]-CR[k] >=0 );
		   model.addConstr(5*(1-U[k])-1+CL[k]+CR[k] >=0 );
	   }

	   // the middle/right bounding element is not to the left of itself, (14)
	   for(int k=0; k < maxDis; k++){
	   		   for(int i=0; i < genomSize; i++){
	   			   model.addConstr(1-BM[k][i]-LM[k][i] >= 0);
	   			   model.addConstr(1-BR[k][i]-LR[k][i] >= 0);
	   		   }
	   }

	   // all elements which are to the left of the middle / right bounding element -> LM_ik =1 / LR_ik=1, (15)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int e=0; e < genomSize; e++){
				   if(i != e){
					   model.addConstr(1-O[k][e][i]+1-BM[k][i]+LM[k][e]-1 >=0 );
					   model.addConstr(1-O[k][e][i]+1-BR[k][i]+LR[k][e]-1 >=0 );
				   }
			   }
		   }
	   }

	   // all elements which are not to the left of the middle / right bounding element -> LM_ik =0 / LR_ik=0, (16)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int e=0; e < genomSize; e++){
				   if(i != e){
					   model.addConstr(1-O[k][i][e]+1-BM[k][i]-LM[k][e] >=0 );
					   model.addConstr(1-O[k][i][e]+1-BR[k][i]-LR[k][e] >=0 );
				   }
			   }
		   }
	   }

	   // a bounding element is to the right of itself, (18)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   model.addConstr(BL[k][i]-RL[k][i]<=0);
			   model.addConstr(BM[k][i]-RM[k][i]<=0);
		   }
	   }

	   // for all elements e which are to the right of the left/middle bounding elements => RL_ek =1 / RM_ek =1, (19)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int e=0; e < genomSize; e++){
				   if(i != e){
					   model.addConstr( 1-O[k][i][e]+1-BL[k][i]+RL[k][e] -1 >=0 );
					   model.addConstr( 1-O[k][i][e]+1-BM[k][i]+RM[k][e] -1 >=0 );
				   }
			   }
		   }
	   }

	   // for all elements e which are to the left of the left/middle bounding elements => RL_ek =0 / RM_ek =0, (20)
	   for(int k=0; k < maxDis; k++){
		   for(int i=0; i < genomSize; i++){
			   for(int e=0; e < genomSize; e++){
				   if(i != e){
					   model.addConstr( 1-O[k][e][i]+1-BL[k][i]-RL[k][e] >=0 );
					   model.addConstr( 1-O[k][e][i]+1-BM[k][i]-RM[k][e] >=0 );
				   }
			   }
		   }
	   }

	   // if element e is between the left and the middle bounding element => WL_ek=1, (22)
	   for(int k=0; k < maxDis; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( 1-RL[k][e]+1-LM[k][e] + WL[k][e] -1 >= 0 );
		   }
	   }

	   // if element e to the right of the left bounding and not the left of the middle bounding => WL_ek =0, (23)
	   for(int k=0; k < maxDis; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( 1-RL[k][e]+LM[k][e] - WL[k][e] >= 0 );
		   }
	   }

	   // if element e to the left of the middle bounding and not the right of the left bounding => WL_ek =0, (24)
	   for(int k=0; k < maxDis; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( 1-LM[k][e] +RL[k][e] -WL[k][e] >= 0 );
		   }
	   }

	   // if element e is between the middle and the right bounding element => WR_ek =1 , (25)
	   for(int k=0; k < maxDis; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( 1-RM[k][e]+1-LR[k][e]+ WR[k][e]-1 >=0);
		   }
	   }

	   // if element e is right of the middle bound and not left of the right bound => WR_ek =0; (26)
	   for(int k=0; k < maxDis; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( 1-RM[k][e]+LR[k][e]- WR[k][e] >=0);
		   }
	   }

	   // if element e is left of the right bound and not right of the middle bound => WR_ek =0; (27)
	   for(int k=0; k < maxDis; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( 1-LR[k][e]+RM[k][e]- WR[k][e] >=0);
		   }
	   }

	   // if one interval gets inverted and two genes e,f are within the interval, then its orientation gets inverted to => O_efk+1 != O_efk, (28)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   for(int f=0; f < genomSize; f++){
				   if(f != e){
					   model.addConstr( 6*(1-CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])+ O[k+1][e][f]- 1+ O[k][e][f] >=0 );
					   model.addConstr( 6*(1-CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])- O[k+1][e][f]+ 1- O[k][e][f] >=0 );

					   model.addConstr( 6*(1-CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])+ O[k+1][e][f]- 1+ O[k][e][f] >=0 );
					   model.addConstr( 6*(1-CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])- O[k+1][e][f]+ 1- O[k][e][f] >=0 );
				   }
			   }
		   }
	   }

	   // if  interval does not get inverted and two genes e,f are within the interval, then its orientation stays => O_efk+1 = O_efk, (30!)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   for(int f=0; f < genomSize; f++){
				   if(f != e){
					   model.addConstr( 6*(CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])+ O[k+1][e][f]- O[k][e][f] >=0 );
					   model.addConstr( 6*(CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])- O[k+1][e][f]+ O[k][e][f] >=0 );

					   model.addConstr( 6*(CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])+ O[k+1][e][f]- O[k][e][f] >=0 );
					   model.addConstr( 6*(CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])- O[k+1][e][f]+ O[k][e][f] >=0 );
				   }
			   }
		   }
	   }

	   // if gene e(f) is in the left(right) interval, then its orientation gets turned => O_efk+1 != O_efk, (29)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   for(int f=0; f < genomSize; f++){
				   if(f != e){
					   model.addConstr( 6*(1-WL[k][e]) +6*(1 -WR[k][f]) + O[k+1][e][f] -1 + O[k][e][f] >= 0 );
					   model.addConstr( 6*(1-WL[k][e]) +6*(1 -WR[k][f]) - O[k+1][e][f] +1 - O[k][e][f] >= 0 );

					   model.addConstr( 6*(1-WL[k][e]) +6*(1 -WR[k][f]) + O[k+1][f][e] -1 + O[k][f][e] >= 0 );
					   model.addConstr( 6*(1-WL[k][e]) +6*(1 -WR[k][f]) - O[k+1][f][e] +1 - O[k][f][e] >= 0 );
				   }
			   }
		   }
	   }


	   // if gene e is not within left and right interval, then it stays to the left of all former left elements => O_efk+1 = O_efk, (28!)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   for(int f=0; f < genomSize; f++){
				   if(f != e){
					   model.addConstr( WL[k][e] + WR[k][e] + O[k+1][e][f] - O[k][e][f] >= 0 );
					   model.addConstr( WL[k][e] + WR[k][e] - O[k+1][e][f] + O[k][e][f] >= 0 );
				   }
			   }
		   }
	   }

	   // if gene e is not within left and right interval, then it stays to the right of all former right elements => O_efk+1 = O_efk, (29!)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   for(int f=0; f < genomSize; f++){
				   if(f != e){
					   model.addConstr( WL[k][f] + WR[k][f] + O[k+1][e][f] - O[k][e][f] >= 0 );
					   model.addConstr( WL[k][f] + WR[k][f] - O[k+1][e][f] + O[k][e][f] >= 0 );
				   }
			   }
		   }
	   }

	   // if element e is within an interval which gets inverted, then its sign gets changed => S_ek+1 != S_ek, (33)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( 1-CL[k] + 1-WL[k][e] + S[k+1][e] -1 + S[k][e] >= 0 );
			   model.addConstr( 1-CL[k] + 1-WL[k][e] - S[k+1][e] +1 - S[k][e] >= 0 );

			   model.addConstr( 1-CR[k] + 1-WR[k][e] + S[k+1][e] -1 + S[k][e] >= 0 );
			   model.addConstr( 1-CR[k] + 1-WR[k][e] - S[k+1][e] +1 - S[k][e] >= 0 );
		   }
	   }

	   // if element e is not within an interval which gets inverted, then its sign stays the same => S_ek+1 = S_ek, (32!)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( CL[k] + 1-WL[k][e] + S[k+1][e] - S[k][e] >= 0 );
			   model.addConstr( CL[k] + 1-WL[k][e] - S[k+1][e] + S[k][e] >= 0 );

			   model.addConstr( CR[k] + 1-WR[k][e] + S[k+1][e] - S[k][e] >= 0 );
			   model.addConstr( CR[k] + 1-WR[k][e] - S[k+1][e] + S[k][e] >= 0 );
		   }
	   }

	   // if element e is not within any interval, then its sign stays the same => S_ek+1 = S_ek, (33!)
	   for(int k=0; k < maxDis-1; k++){
		   for(int e=0; e < genomSize; e++){
			   model.addConstr( WL[k][e] + WR[k][e] + S[k+1][e] - S[k][e] >= 0 );
			   model.addConstr( WL[k][e] + WR[k][e] - S[k+1][e] + S[k][e] >= 0 );
		   }
	   }

	   // O_iik =0 for all i, k
	   for(int k=0; k< maxDis-1;k++){
		   for(int i=0; i< genomSize; i++){
			   model.addConstr(O[k+1][i][i] - O[k][i][i]<=0);
			   model.addConstr(O[k+1][i][i] - O[k][i][i]>=0);
		   }
	   }


//////////////////////////////////////////////////////////////////// Optimize model /////////////////////////////////////////////////////

	    model.optimize();

//////////////////////////////////////////////////////////////////// OUTPUT ////////////////////////////////////////////////////////////

	    //store the objective value in objV
	    objV=model.get(GRB_DoubleAttr_ObjVal);

	    // contains the sum of ones of a row of O
	    int* rowSum = new int[genomSize];

	    if(output){
	    	for(int k=0; k<maxDis-1; k++){
				cout << "_________________" << "k=" << k << "__________________" << endl;
				cout << "genome: " << endl;
				 // counts the sum of every row
				for(int i=0; i< genomSize; i++){
					rowSum[i]=0;
					for(int j=0; j < genomSize; j++){
						rowSum[i] = rowSum[i]+O[k][i][j].get(GRB_DoubleAttr_X);
					}
				}
				// gives the right genorder out
				for(int i=0; i< genomSize; i++){
					for(int j=0; j <genomSize; j++){
						if(rowSum[j]==genomSize-1-i){
							if(S[k][j].get(GRB_DoubleAttr_X) == 1){
								cout << (-1)*(j) << "\t";
							}
							else{
								cout << j << "\t";
							}
						}
					}
				}
				cout << endl;

				cout << "O: " << endl;
				for(int i=0; i<genomSize; i++){
					for(int j=0; j< genomSize;j++){
						if(j != genomSize-1){
							cout << O[k][i][j].get(GRB_DoubleAttr_X) << " ";
						}
						else{
							cout << O[k][i][j].get(GRB_DoubleAttr_X) << endl;
						}
					}
				}

				cout << "BL: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << BL[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "BM: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << BM[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "BR: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << BR[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "WL: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << WL[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "WR: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << WR[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "LM: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << LM[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "LR: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << LR[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "RL: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << RL[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "RM: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << RM[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "S: " << endl;
				for(int i=0; i<genomSize; i++){
					cout << S[k][i].get(GRB_DoubleAttr_X) << " ";
				}
				cout << endl;

				cout << "I:" << "\t" << "T:" << "\t" <<  "iT:" << endl;
				cout << I[k].get(GRB_DoubleAttr_X) << "\t" << T[k].get(GRB_DoubleAttr_X) << "\t" <<  U[k].get(GRB_DoubleAttr_X) << endl;

				cout << "CL:" << "\t" << "CR:" << endl;
				cout << CL[k].get(GRB_DoubleAttr_X) << "\t" << CR[k].get(GRB_DoubleAttr_X) << endl;
	    	}
	    }



	    	//////////////////// calculate the rrrmt scenario //////////////////////////////

	    		    int* nrOnes;
	    			nrOnes = new int[genomSize];
	    		    int leftBound=0;
	    		    int middleBound=0;
	    		    int rightBound=0;

	    		    for(int k=0; k<maxDis-1;k++){
	    		    	// reset the settings for every k
	    		    	leftBound=0;
	    		    	middleBound=0;
	    		    	rightBound=0;
	    		    	for(int m=0; m<genomSize; m++){
	    		    		nrOnes[m]=0;
	    		    	}

	    		    	// calculate how many ones are in every column of O[k]
	    				for(int col=0; col< genomSize; col++){
	    					for(int line=0; line< genomSize;line++){
	    		    			nrOnes[col] += O[k][line][col].get(GRB_DoubleAttr_X);
	    		    		}
	    		    	}

	    		    	// case inversion
	    		    	if(I[k].get(GRB_DoubleAttr_X)==1){
	    		    		//	build inversion(rev) rrrmt
	    		    		for(int i=0; i<genomSize; i++){
	    		    			if(BL[k][i].get(GRB_DoubleAttr_X)==1){
	    	    					leftBound=nrOnes[i];
	    		    			}
	    		    			if(BR[k][i].get(GRB_DoubleAttr_X)==1){
	    							rightBound=nrOnes[i]-1;
	    						}
	    		    		}
	    		    		middleBound =-1;
	    		    		//define inversion
	    		    		rrrmt *r;
	    		    		//cout << "before: " << cpg1 << endl;
	    		    		r = new rev( leftBound, rightBound, cpg1 );
	    		    		r->apply(cpg1);
	    		    		//cout << leftBound << " " << rightBound << endl;
	    		    		//cout << "after: " << cpg1 << endl;
	    		    		// add inversion to the scenario
	    		    		push_back(r);
	    		    	}

	    		    	// case transposition
	    		    	if(T[k].get(GRB_DoubleAttr_X)==1){
	    		    		//	build transposition(transp) rrrmt
	    					for(int i=0; i<genomSize; i++){
	    						if(BL[k][i].get(GRB_DoubleAttr_X)==1){
	    							leftBound=nrOnes[i];
	    						}
	    						if(BM[k][i].get(GRB_DoubleAttr_X)==1){
	    							middleBound=nrOnes[i];
	    						}
	    						if(BR[k][i].get(GRB_DoubleAttr_X)==1){
	    							rightBound=nrOnes[i];
	    						}
	    					}
	    					//define transposition
	    					rrrmt *t;
	    					t = new transp( leftBound, middleBound, rightBound, cpg1 );
	    					t->apply(cpg1);
	    		    		//cout << leftBound << " " << middleBound << " " << rightBound << endl;

	    					// add transposition to the scenario
	    					push_back(t);
	    		    	}

	    		    	// case inverse transposition
	    		    	if(U[k].get(GRB_DoubleAttr_X)==1){
	    		    		//	build inverse transposition(revtransp) rrrmt
	    					if(CL[k].get(GRB_DoubleAttr_X)==1){
	    						for(int i=0; i<genomSize; i++){
	    							if(BL[k][i].get(GRB_DoubleAttr_X)==1){
	    								leftBound=nrOnes[i];
	    							}
	    							if(BM[k][i].get(GRB_DoubleAttr_X)==1){
	    								middleBound=nrOnes[i]-1;
	    							}
	    							if(BR[k][i].get(GRB_DoubleAttr_X)==1){
	    								rightBound=nrOnes[i];
	    							}
	    						}

	    						//define inverse transposition
	    						rrrmt *it;

	    						// left sequence gets inverted
	    						it = new revtransp( leftBound, middleBound, rightBound, cpg1 );
	    						it->apply(cpg1);
	    						//cout << leftBound << " " << middleBound << " " << rightBound << endl;
	    						// add inverse transposition to the scenario
	    						push_back(it);
	    					}
	    					if(CR[k].get(GRB_DoubleAttr_X)==1){
	    						for(int i=0; i<genomSize; i++){
	    							if(BL[k][i].get(GRB_DoubleAttr_X)==1){
	    								leftBound=nrOnes[i];
	    							}
	    							if(BM[k][i].get(GRB_DoubleAttr_X)==1){
	    								middleBound=nrOnes[i];
	    							}
	    							if(BR[k][i].get(GRB_DoubleAttr_X)==1){
	    								rightBound=nrOnes[i]-1;
	    							}
	    						}
	    						//define inverse transposition
	    						rrrmt *it;
	    						// right sequence gets inverted
	    						it = new revtransp( middleBound, rightBound, leftBound, cpg1 );
	    						it->apply(cpg1);
	    						//cout << middleBound << " " << rightBound << " " << leftBound << endl;
	    						// add inverse transposition to the scenario
	    						push_back(it);
	    					}
	    		    	}
	    		    }
	    	//break maxDis
			break;

	  } catch(GRBException e) {
	    cerr << "Error code = " << e.getErrorCode() << endl;
	    cerr << e.getMessage() << endl;
	  } catch(...) {
	    cerr << "Exception during optimization" << endl;
	    break;
	  }
	}
////////////////////////////////////////////////////////////////////////////////////////
	 //cout << "/////////////////////////////////////////////////////////////////////////////////////////////////////" << endl;


	 //int n;	// length of the genomes
	 //vector<int> pi; // renamed genome, s.t. g2 = iota
	//genom gpi; // contains the names of the renamed genomes
//	genom cg1 = g1;
//	rrrmt *it;
//	cout << cg1<< endl;
//	it=new rev(3,3,g1);
//	it->apply(cg1);
//	cout << cg1 << endl;
	//n = g1.size();

	//cout << g1 << endl;
	//cout << g2 << endl;

	// identify returns the renamed genome, s.t. we only have to sort g1
	//pi = g2.identify( g1 );

	//copy( pi.begin(), pi.end(), ostream_iterator<int>(cout, " ") );
	//cout << endl;

	// identify_g returns the names of the genes of the renamed genome
	//gpi = g2.identify_g(g1);

	// gpi.set_nmap( NULL );
	//cout << gpi <<endl;

	//r = new rev( 1, 3, cg1 );
	//cout << "huhu" << (*cst)[r] << endl;

	//cout << cg1 << endl;
	//r->apply( cg1 );
	//this->insert( r );
	//cout << cg1 << endl;

}*/

ilp_rrrmt::ilp_rrrmt(const ilp_rrrmt &c ) : ordered(c){}

ilp_rrrmt::~ilp_rrrmt(){}

ilp_rrrmt::ilp_rrrmt(const genom &g1,
						const genom &g2,
						const costfoo *cst,
						double &objV,
						float &ms,
						rrrmt* &mr,
						bool &optimal,
						bool &is_cutoff,
						bool &time_exceeded,
						vector<alternative> &alternative_scen,
						double time_limit,
						const vector<vector<float> > &cdps,
						const vector<vector<rrrmt*> > &cdpr,
						string name,
						unsigned int all,
						bool output,
						const vector<vector<int> > &mapping,
						const genom &origin,
						bool useCrex2,
						bool use_cutoff,
						float cutoff,
						bool fixed_numb_rrrmt,
						unsigned int exact_distance){
	if(g1==g2){
		//both genomes are equal
		rrrmt * sol_sce;
		sol_sce= new emptyscen();
		this->push_back(sol_sce->clone());
		delete sol_sce;
	}else{
		if(use_cutoff){
			if(cutoff<=0){
				cerr << "Cutoff should be positive! --> exiting!" << endl;
				cerr << "cutoff: " << cutoff << endl;
				exit(EXIT_FAILURE);
			}
		}
		if(time_limit<=0){
			cerr << "Time limit should be positive! --> exiting!" << endl;
			cerr << "time limit: " << time_limit << endl;
			exit(EXIT_FAILURE);
		}
		// only used for useCrex2==true
		vector<float> *objectValues;			//contains the objective values of every ilp + child costs
		objectValues = new vector<float>;

		vector<unordered*> *objectUnordered;	//contains all ILP scenarios + child scenarios
		objectUnordered =new vector<unordered*>;

		vector<ordered*> *objectOrdered;		//contains all ILP scenarios
		objectOrdered =new vector<ordered*>;

		float currentBest=std::numeric_limits<int>::max();		//stores the current best objective value, used for setting cutoffs

		genom cpg1 = g1;
		//	get size of the genomes
		int genomSize;
		genomSize = g1.size()+1;

		// test: do both genomes hae the same size
		if(g2.size()!=genomSize-1){
			cout << "Both genomes do not have the same size!" << endl;
		}

		// identify_g returns the names of the genes of the renamed genome
		genom gpi; 				// contains the names of the renamed genomes
		gpi = g2.identify_g(g1);

		bool firstSol=false;


		// only used for useCrex2 == true
		// change costs if the sign of qperm = g1 and renamed qperm = gpi is changed
		vector<vector<float> > copy_cdps=cdps;
		vector<vector<rrrmt*> > copy_cdpr=cdpr;

		if(useCrex2){
			for (int i=0; i<g1.size();i++){
				bool bothMinus = g1[i] < 0 && gpi[i] < 0;
				bool bothPlus = g1[i] > 0 && gpi[i] > 0;
				if( bothMinus || bothPlus ){
					//do nothing
				}else{
					// exchange costs
					copy_cdps[i][INC]=cdps[i][DEC];
					copy_cdps[i][DEC]=cdps[i][INC];
					//exchange scenarios
					copy_cdpr[i][INC]=cdpr[i][DEC];
					copy_cdpr[i][DEC]=cdpr[i][INC];
				}
			}
		}
		// get costs of rrrmt events
		rrrmt *i;
		i = new rev( 1, 2, gpi );
		float costI = (*cst)[i];
		rrrmt *iT;
		iT = new revtransp( 1, 2, 3, gpi );
		float costIT = (*cst)[iT];
		rrrmt *T;
		T = new transp(1,2,3, gpi);
		float costT = (*cst)[T];
		rrrmt *tdl;
		tdl =new tdrl(gpi);
		float costTDRL = (*cst)[tdl];

		// find rrrmt with lowest cost value
		float minimumCosts = min(min(costTDRL,costIT), min(costI,costT));

		// find maximum cost value
		double maximumCosts = max(max(costTDRL,costIT), max(costI,costT));

		if(use_cutoff){
			cutoff = cutoff - minimumCosts/4;
		}

		// check all cost constraints mention in paper: "On Weighting Schemes for Gene Order Analysis" since this gives us the possibility to
		// reduce a bunch of events

		bool noAllEvents=false;
		bool removeI=false;
		bool removeT=false;
		bool removeIT=false;
		// T = I + IT (1)
		if(costT > costI + costIT){
			removeT=true;
			noAllEvents=true;
		}

		// T = 3*I (2)
		if(costT > 3*costI){
			removeT=true;
			noAllEvents=true;
		}

		// T = 2*IT (3)
		if(costT > 2*costIT){
			removeT=true;
			noAllEvents=true;
		}

		// IT = I + T (4)
		if(costIT > costI + costT){
			removeIT=true;
			noAllEvents=true;
		}

		// IT = 2*I (5)
		if(costIT > 2*costI){
			removeIT=true;
			noAllEvents=true;
		}

		// I = T + IT (6)
		if(costI > costT + costIT){
			removeI=true;
			noAllEvents=true;
		}

		// I = 3*IT (7)
		if(costI > 3*costIT){
			removeI=true;
			noAllEvents=true;
		}

		// only used for useCrex2 == true
		// calculate minimum and maximum child cost
		int min_cost=0;
		int max_cost=0;

		if(useCrex2){
			for(int i=0; i<cdps.size();i++){
				if(cdps[i][INC]<=cdps[i][DEC]){
					min_cost += cdps[i][INC];
					max_cost += cdps[i][DEC];
				}else{
					min_cost += cdps[i][DEC];
					max_cost += cdps[i][INC];
				}
			}
		}

		int upperBound;		// upper bound for the number of rrrmt for the given scenario

		// contains O matrix of the start genome
		int** genomStart = new int*[genomSize];
		for(int i = 0; i < genomSize; i++){
			genomStart[i] = new int[genomSize];
		}

		for(int i=0; i< genomSize; i++){
			for(int j=0; j < genomSize; j++){
				genomStart[i][j]=0;
			}
		}

		// temp variables
		int tempGPiI;
		int tempGPiJ;

		for(int i=0; i<genomSize-1; i++){
			for(int j=0; j<genomSize-1; j++){
				tempGPiI = abs( gpi[i] );
				tempGPiJ = abs( gpi[j] );
				if(i<j){
					genomStart[tempGPiI-1][tempGPiJ-1]=1;
				}else if(i>j){
					genomStart[tempGPiJ-1][tempGPiI-1]=1;
				}
			}
		}

		// sets genomStart for the dummy element
		for(int i=0; i<genomSize; i++){
			genomStart[genomSize-1][i]=0;
			if(i!=genomSize-1){
				genomStart[i][genomSize-1]=1;
			}
		}

///////////////	add ILP //////////////////////////////////////////////////////////////////////////////////////////////
		int maxDis=1;
		double last_scenario_cost=std::numeric_limits<double>::max();
		do{
			//maxDis: number of used rrrmt-1 of the scenario (since scenario g1==g2 is already excluded it starts with distance 1 <=> maxDis==2)

			maxDis++;
			//rrrmt is only calculated for a specific number of rrrmts
			if(fixed_numb_rrrmt){
				maxDis=exact_distance+1;
			}

			try {
				// create gurobi environement
				GRBEnv env = GRBEnv();
				// produce output ??
				if(!output){
					env.set(GRB_IntParam_OutputFlag,0);
				}
				// only use specific number of threads
				env.set(GRB_IntParam_Threads, 1);

				// create gurobi model
				GRBModel model = GRBModel(env);

				////////****** cutoffs seems to be buggy in GUROBI
				//sets cutoff given to function, might be overwritten by next function
				//if(use_cutoff && cutoff < model.get(GRB_DoubleParam_Cutoff)){
				//	model.set(GRB_DoubleParam_Cutoff,cutoff);
					//use_cutoff=false;
				//	cout << "set cutoff initial to: " << cutoff << endl;
				//}

				// only find solutions better than previous found solutions -> set cutoff
				//if(!objectValues->empty() && currentBest < model.get(GRB_DoubleParam_Cutoff)){
				//	model.set(GRB_DoubleParam_Cutoff,currentBest);
				//	cout << "set cutoff iterative to: " << currentBest << endl;
				//}

				// set time limit
				model.set(GRB_DoubleParam_TimeLimit, time_limit);



				//TODO
				// set gap
				//model.getEnv().set(GRB_DoubleParam_MIPGap, minimumCosts-0.01);
//////////////////////////////// Create variables ////////////////////////////////////////////////////////////////////////

				// O_ijk stores the permutation
				GRBVar*** O;
				O = new GRBVar**[maxDis];
				for( int k=0; k < maxDis; k++){
					O[k] = new GRBVar*[genomSize];
					for(int i=0; i< genomSize; i++){
						O[k][i] = new GRBVar[genomSize];
					}
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						for(int j=0; j< genomSize; j++){
							O[k][i][j] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
						}
					}
				}

				// S_ik contains the sign of a gene, S_ik = 1 <-> gene i has a negative sign
				GRBVar** S;
				S = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					S[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						S[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// I_k = 1 <=> pi_k was changed by an inversion to pi_(k+1)
				GRBVar* I;
				I = new GRBVar[maxDis];
				for(int k=0; k<maxDis; k++){
						I[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
				}

				// T_k = 1 <=> pi_k was changed by a transposition to pi_(k+1)
				GRBVar* T;
				T = new GRBVar[maxDis];
				for(int k=0; k<maxDis; k++){
						T[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
				}

				// U_k = 1 <=> pi_k was changed by an inverse transposition to pi_(k+1)
				GRBVar* U;
				U = new GRBVar[maxDis];
				for(int k=0; k<maxDis; k++){
						U[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
				}

				//TODO
				// TDRL_k = 1 <=> pi_k was changed by an TDRL to pi_(k+1)
				GRBVar* TDL;
				TDL = new GRBVar[maxDis];
				for(int k=0; k<maxDis; k++){
					TDL[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
				}

				// FS_ki ==0 <=> pi_k(i) \in F ;  FS_ki ==1 <=> pi_k(i) \in S ; only for TDRLs
				GRBVar** FS;
				FS = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					FS[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						FS[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// BL_ik left breakpoint variable
				GRBVar** BL;
				BL = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					BL[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						BL[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// BM_ik middle breakpoint variable
				GRBVar** BM;
				BM = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					BM[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						BM[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// BR_ik right breakpoint variable
				GRBVar** BR;
				BR = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					BR[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						BR[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// CL_k = 1 <=> the interval between BL and BM gets inverted
				GRBVar* CL;
				CL = new GRBVar[maxDis];
				for(int k=0; k<maxDis; k++){
					CL[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
				}

				// CR_k = 1 <=> the interval between BM and BR gets inverted
				GRBVar* CR;
				CR = new GRBVar[maxDis];
				for(int k=0; k<maxDis; k++){
					CR[k] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
				}

				// WL_ek =1 <=> element e is within the left interval
				GRBVar** WL;
				WL = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					WL[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						WL[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// WR_ek =1 <=> element e is within the right interval
				GRBVar** WR;
				WR = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					WR[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						WR[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// LM_ek =1 <=> element e is to the left of the middle bonding element
				GRBVar** LM;
				LM = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					LM[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						LM[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// LR_ek =1 <=> element e is to the left of the right bonding element
				GRBVar** LR;
				LR = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					LR[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						LR[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// RL_ek =1 <=> element e is to the right of the left bonding element (including bounding element)
				GRBVar** RL;
				RL = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					RL[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						RL[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				// RM_ek =1 <=> element e is to the right of the middle bonding element (including bounding element)
				GRBVar** RM;
				RM = new GRBVar*[maxDis];
				for( int k=0; k< maxDis; k++){
					RM[k]= new GRBVar[genomSize];
				}

				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize; i++){
						RM[k][i] = model.addVar(0.0, 1.0, 0.0, GRB_BINARY);
					}
				}

				model.update();

				// Set objective:
				GRBLinExpr obj = 0;
				for(int k =0 ; k< maxDis-1; k++){
					//////////////////////////////////////TODO////////////////////////////////////////
					obj += I[k]*costI + T[k]*costT + U[k]*costIT + TDL[k]*costTDRL;
				}

				if(useCrex2){
					// add costs of children
					// child costs are added according to the sign of the used rrrmt
					for(int i=0; i<cdps.size();i++){
						obj += (1-S[0][abs(gpi[i])-1])*copy_cdps[i][INC] + S[0][abs(gpi[i])-1]*copy_cdps[i][DEC];
					}
				}

				model.setObjective(obj, GRB_MINIMIZE);

				if(use_cutoff){
					model.addConstr(obj-cutoff<=0);
					if(currentBest < cutoff){
						model.addConstr(obj-currentBest<=0);
					}
				}

///////////////////////////////////////Add constraints////////////////////////////////////////////////////////////

				// Add initial constraints for the starting & target genome, (1)
				for(int i=0; i< genomSize;i++){
					for(int j=0; j< genomSize; j++){
						// starting genome
						model.addConstr( O[0][i][j] - genomStart[i][j] <= 0 );
						model.addConstr( O[0][i][j] - genomStart[i][j] >= 0 );
						// target genome
						if(i>j){
							model.addConstr( O[maxDis-1][i][j] <= 0);
							model.addConstr( O[maxDis-1][i][j] >= 0);
						}
						if(i<j){
							model.addConstr( O[maxDis-1][i][j] -1 <= 0);
							model.addConstr( O[maxDis-1][i][j] -1 >= 0);
						}
					}
				}

				// Add initial constraints for the signs of starting & target genome (based on genes), (2)
				for(int i=0; i<genomSize-1;i++){
					// starting genome signs
					if(gpi[i]<0){
						if(!useCrex2){
							model.addConstr( S[0][ (gpi[i]*(-1)) -1] -1 <= 0);
							model.addConstr( S[0][ (gpi[i]*(-1)) -1] -1 >= 0);
						}
					}
					else{
						if(!useCrex2){
							model.addConstr( S[0][ gpi[i]-1 ] <= 0);
							model.addConstr( S[0][ gpi[i]-1 ] >= 0);
						}
					}
					// target genome signs
					model.addConstr(S[maxDis-1][i] <= 0);
					model.addConstr(S[maxDis-1][i] >= 0);

				}

				// sets O_ijk variables for the dummy element
				for(int k=0; k<maxDis; k++){
					for(int i=0; i<genomSize;i++){
						model.addConstr(O[k][genomSize-1][i]>=0);
						model.addConstr(O[k][genomSize-1][i]<=0);
						if(i!=genomSize-1){
							model.addConstr(O[k][i][genomSize-1]<=1);
							model.addConstr(O[k][i][genomSize-1]>=1);
						}
					}
				}

				//sets signs of dummy element
				for(int k=0; k<maxDis; k++){
					model.addConstr(S[k][genomSize-1]>=0);
					model.addConstr(S[k][genomSize-1]<=0);
				}

			   // Add constraints, that only one rearrangement at step k is allowed, (4)
			   for(int k=0; k<maxDis; k++){
				   // there must be one operation
				   model.addConstr(I[k]+T[k]+U[k]+TDL[k] -1 >= 0);
				   // TDRL=1 => T,iT,I=0
				   model.addConstr(1 - TDL[k] - I[k] >= 0);
				   model.addConstr(1 - TDL[k] - T[k] >= 0);
				   model.addConstr(1 - TDL[k] - U[k] >= 0);
				   // T=1 => I,U,TDRL=0
				   model.addConstr(1 - T[k] - I[k] >= 0);
				   model.addConstr(1 - T[k] - TDL[k] >= 0);
				   model.addConstr(1 - T[k] - U[k] >= 0);
				   // I=1 => T,U,TDRL=0
				   model.addConstr(1 - I[k] - T[k] >= 0);
				   model.addConstr(1 - I[k] - TDL[k] >= 0);
				   model.addConstr(1 - I[k] - U[k] >= 0);
				   // U=1 => T,I,TDRL=0
				   model.addConstr(1 - U[k] - T[k] >= 0);
				   model.addConstr(1 - U[k] - TDL[k] >= 0);
				   model.addConstr(1 - U[k] - I[k] >= 0);
			   }

			   //TODO TDRL=1 => sign change = 0
			   for(int k=0; k<maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr( (1-TDL[k]) + S[k+1][e] - S[k][e] >= 0 );
					   model.addConstr( (1-TDL[k]) + S[k][e] - S[k+1][e] >= 0 );
				   }
			   }
//			   //TODO number of TDRL is limited to 1
//			   for(int k=0;k<maxDis-1; k++){
//				   for(int k2=0; k2<maxDis-1; k2++){
//					   if(k!=k2){
//						   model.addConstr( 1-TDL[k] - TDL[k2] >=0 );
//					   }
//				   }
//			   }

			   //TODO TDRL=1 && FS_ke==FS_kf => O_kef = O_k+1ef && O_kfe = O_k+1fe
			   for(int k=0; k<maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   for(int f=0; f < genomSize; f++){
						   model.addConstr( (1-TDL[k]) + (1-FS[k][e]) +(1-FS[k][f]) + O[k][e][f] - O[k+1][e][f] >= 0 );
						   model.addConstr( (1-TDL[k]) + (1-FS[k][e]) +(1-FS[k][f]) + O[k+1][e][f] - O[k][e][f] >= 0 );

						   model.addConstr( (1-TDL[k]) + FS[k][e] + FS[k][f] + O[k][e][f] - O[k+1][e][f] >= 0 );
						   model.addConstr( (1-TDL[k]) + FS[k][e] + FS[k][f] + O[k+1][e][f] - O[k][e][f] >= 0 );
					   }
				   }
			   }

			   //TODO TDRL=1 && FS_ek != FS_ => O_kef != O_k+1ef && O_kfe = O_k+1fe
			   for(int k=0; k<maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   for(int f=0; f < genomSize; f++){
						   //TDRL=1 && FS_ek=0 && FS_fk=1 => O_k+1ef=1
						   model.addConstr( (1-TDL[k]) + FS[k][e] + (1-FS[k][f]) + O[k+1][e][f] - 1 >= 0 );

						   //TDRL=1 && FS_ek=1 && FS_fk=0 => O_k+1ef=0
						   model.addConstr( (1-TDL[k]) + (1-FS[k][e]) + FS[k][f] - O[k+1][e][f]  >= 0 );
					   }
				   }
			   }

			   // in every genome, there exists only one BL, BM, BR, (5)
			   GRBLinExpr tempBL = 0;
			   GRBLinExpr tempBM = 0;
			   GRBLinExpr tempBR = 0;
			   for(int k=0; k < maxDis; k++){
				   tempBL = 0;
				   tempBM = 0;
				   tempBR = 0;
				   for(int i=0; i < genomSize; i++){
					   tempBL += BL[k][i];
					   tempBM += BM[k][i];
					   tempBR += BR[k][i];
				   }
				   model.addConstr(tempBL -1 <=0);
				   model.addConstr(tempBL -1 >=0);
				   model.addConstr(tempBM -1 <=0);
				   model.addConstr(tempBM -1 >=0);
				   model.addConstr(tempBR -1 <=0);
				   model.addConstr(tempBR -1 >=0);
			   }

			   // TODO constraints that satisfy that a parsimonious rrrmt does not contain a cut
			   GRBLinExpr c_sum = 0;
			   GRBLinExpr c_prim_sum = 0;
			   for(int k=0; k< maxDis; k++){
				   for(int i=0; i<genomSize-1; i++){
					   c_sum = 0;
					   c_prim_sum = 0;
					   for(int c=0; c< genomSize; c++){
						   c_sum += O[k][c][i];
						   c_prim_sum += O[k][c][i+1];
					   }
					   //case ...i i+1... in pi

					   //breakpoints:
					   model.addConstr(100*(1-O[k][i][i+1]) - 2*(1-(c_prim_sum-c_sum)) + 100*S[k][i] + 100*S[k][i+1] - BL[k][i+1] >= 0);
					   model.addConstr(100*(1-O[k][i][i+1]) - 2*(1-(c_prim_sum-c_sum)) + 100*S[k][i] + 100*S[k][i+1] - BM[k][i+1] >= 0);
					   model.addConstr(100*(1-O[k][i][i+1]) - 2*(1-(c_prim_sum-c_sum)) + 100*S[k][i] + 100*S[k][i+1] - BR[k][i+1] >= 0);

					   //FS:
					   model.addConstr(100*(1-O[k][i][i+1]) - 2*(1-(c_prim_sum-c_sum)) + 100*S[k][i] + 100*S[k][i+1] + FS[k][i] - FS[k][i+1] >= 0);
					   model.addConstr(100*(1-O[k][i][i+1]) - 2*(1-(c_prim_sum-c_sum)) + 100*S[k][i] + 100*S[k][i+1] + FS[k][i+1] - FS[k][i] >= 0);

					   // case ... -(i+1) -i ...

					   //breakpoints
					   model.addConstr(100*(O[k][i][i+1]) - 2*(1-(c_sum-c_prim_sum)) + 100*(1-S[k][i]) + 100*(1-S[k][i+1]) - BL[k][i] >= 0);
					   model.addConstr(100*(O[k][i][i+1]) - 2*(1-(c_sum-c_prim_sum)) + 100*(1-S[k][i]) + 100*(1-S[k][i+1]) - BM[k][i] >= 0);
					   model.addConstr(100*(O[k][i][i+1]) - 2*(1-(c_sum-c_prim_sum)) + 100*(1-S[k][i]) + 100*(1-S[k][i+1]) - BR[k][i] >= 0);

					   //FS:
					   model.addConstr(100*(O[k][i][i+1]) - 2*(1-(c_sum-c_prim_sum)) + 100*(1-S[k][i]) + 100*(1-S[k][i+1]) + FS[k][i] - FS[k][i+1] >= 0);
					   model.addConstr(100*(O[k][i][i+1]) - 2*(1-(c_sum-c_prim_sum)) + 100*(1-S[k][i]) + 100*(1-S[k][i+1]) + FS[k][i+1] - FS[k][i] >= 0);
				   }
			   }


			   // left bounding element has to be left of the middle one, (6)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int j=0; j < genomSize; j++){
							  model.addConstr(50*TDL[k] + 5*(1-BL[k][i])+ 5*(1-BM[k][j]) + O[k][i][j] -1 >=0);
					   }
				   }
			   }

			   // for transpositions the middle bound must be left of the right bound, (7)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int j=0; j < genomSize; j++){
							   model.addConstr(50*(1-T[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) + O[k][i][j] -1 >=0);
					   }
				   }
			   }

			   // for inverse transpositions the middle bound must be left of the right bound, (8)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int j=0; j < genomSize; j++){
							   model.addConstr(50*(1-U[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) + O[k][i][j] -1 >=0);
					   }
				   }
			   }

			   // for inversion the middle and the right bound are identical, (9)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int j=0; j < genomSize; j++){
						   model.addConstr(50*(1-I[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) + O[k][i][j] -O[k][j][i] >=0);
						   model.addConstr(50*(1-I[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) - O[k][i][j] +O[k][j][i] >=0);
					   }
				   }
			   }

			   // if I_k =1 => the left interval gets turned and the right interval not, thus CL_k=1 and CR_k=0, (11)
			   for(int k=0; k<maxDis; k++){
				   model.addConstr(I[k]-CL[k] <=0);
				   model.addConstr(1-I[k]-CR[k] >=0);
			   }

			   // if T_k=1 => no interval gets inverted, thus CL_k=CR_k=0, (12)
			   for(int k=0; k<maxDis; k++){
				   model.addConstr(50*(1-T[k])-CL[k]-CR[k] >=0 );
			   }

			   // if U_k=1 => exactly one of both intervals gets inverted, (13)
			   for(int k=0; k<maxDis; k++){
				   model.addConstr(50*(1-U[k])+1-CL[k]-CR[k] >=0 );
				   model.addConstr(50*(1-U[k])-1+CL[k]+CR[k] >=0 );
			   }

			   // the middle/right bounding element is not to the left of itself, (14)
			   for(int k=0; k < maxDis; k++){
					   for(int i=0; i < genomSize; i++){
						   model.addConstr(50*TDL[k] + 1-BM[k][i]-LM[k][i] >= 0);
						   model.addConstr(50*TDL[k] + 1-BR[k][i]-LR[k][i] >= 0);
					   }
			   }

			   // all elements which are to the left of the middle / right bounding element -> LM_ik =1 / LR_ik=1, (15)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int e=0; e < genomSize; e++){
						   if(i != e){
							   model.addConstr(50*TDL[k] + 1-O[k][e][i]+1-BM[k][i]+LM[k][e]-1 >=0 );
							   model.addConstr(50*TDL[k] + 1-O[k][e][i]+1-BR[k][i]+LR[k][e]-1 >=0 );
						   }
					   }
				   }
			   }

			   // all elements which are not to the left of the middle / right bounding element -> LM_ik =0 / LR_ik=0, (16)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int e=0; e < genomSize; e++){
						   if(i != e){
							   model.addConstr(50*TDL[k] + 1-O[k][i][e]+1-BM[k][i]-LM[k][e] >=0 );
							   model.addConstr(50*TDL[k] + 1-O[k][i][e]+1-BR[k][i]-LR[k][e] >=0 );
						   }
					   }
				   }
			   }

			   // a bounding element is to the right of itself, (18)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   model.addConstr(-5*TDL[k] + BL[k][i]-RL[k][i]<=0);
					   model.addConstr(-5*TDL[k] + BM[k][i]-RM[k][i]<=0);
				   }
			   }

			   // for all elements e which are to the right of the left/middle bounding elements => RL_ek =1 / RM_ek =1, (19)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int e=0; e < genomSize; e++){
						   if(i != e){
							   model.addConstr(5*TDL[k] + 1-O[k][i][e]+1-BL[k][i]+RL[k][e] -1 >=0 );
							   model.addConstr(5*TDL[k] + 1-O[k][i][e]+1-BM[k][i]+RM[k][e] -1 >=0 );
						   }
					   }
				   }
			   }

			   // for all elements e which are to the left of the left/middle bounding elements => RL_ek =0 / RM_ek =0, (20)
			   for(int k=0; k < maxDis; k++){
				   for(int i=0; i < genomSize; i++){
					   for(int e=0; e < genomSize; e++){
						   if(i != e){
							   model.addConstr(5*TDL[k] + 1-O[k][e][i]+1-BL[k][i]-RL[k][e] >=0 );
							   model.addConstr(5*TDL[k] + 1-O[k][e][i]+1-BM[k][i]-RM[k][e] >=0 );
						   }
					   }
				   }
			   }

			   // if element e is between the left and the middle bounding element => WL_ek=1, (22)
			   for(int k=0; k < maxDis; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + 1-RL[k][e] + 1-LM[k][e] + WL[k][e] -1 >= 0 );
				   }
			   }

			   // if element e to the right of the left bounding and not the left of the middle bounding => WL_ek =0, (23)
			   for(int k=0; k < maxDis; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + 1-RL[k][e] + LM[k][e] - WL[k][e] >= 0 );
				   }
			   }

			   // if element e to the left of the middle bounding and not the right of the left bounding => WL_ek =0, (24)
			   for(int k=0; k < maxDis; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + 1-LM[k][e] +RL[k][e] -WL[k][e] >= 0 );
				   }
			   }

			   // if element e is between the middle and the right bounding element => WR_ek =1 , (25)
			   for(int k=0; k < maxDis; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + 1-RM[k][e]+1-LR[k][e]+ WR[k][e]-1 >=0);
				   }
			   }

			   // if element e is right of the middle bound and not left of the right bound => WR_ek =0; (26)
			   for(int k=0; k < maxDis; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + 1-RM[k][e]+LR[k][e]- WR[k][e] >=0);
				   }
			   }

			   // if element e is left of the right bound and not right of the middle bound => WR_ek =0; (27)
			   for(int k=0; k < maxDis; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + 1-LR[k][e]+RM[k][e]- WR[k][e] >=0);
				   }
			   }

			   // if one interval gets inverted and two genes e,f are within the interval, then its orientation gets inverted to => O_efk+1 != O_efk, (28)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   for(int f=0; f < genomSize; f++){
						   if(f != e){
							   model.addConstr(50*TDL[k] + 6*(1-CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])+ O[k+1][e][f]- 1+ O[k][e][f] >=0 );
							   model.addConstr(50*TDL[k] + 6*(1-CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])- O[k+1][e][f]+ 1- O[k][e][f] >=0 );

							   model.addConstr(50*TDL[k] + 6*(1-CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])+ O[k+1][e][f]- 1+ O[k][e][f] >=0 );
							   model.addConstr(50*TDL[k] + 6*(1-CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])- O[k+1][e][f]+ 1- O[k][e][f] >=0 );
						   }
					   }
				   }
			   }


			   // if  interval does not get inverted and two genes e,f are within the interval, then its orientation stays => O_efk+1 = O_efk, (30!)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   for(int f=0; f < genomSize; f++){
						   if(f != e){
							   model.addConstr(50*TDL[k] + 6*(CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])+ O[k+1][e][f]- O[k][e][f] >=0 );
							   model.addConstr(50*TDL[k] + 6*(CL[k]) + 6*(1-WL[k][e])+ 6*(1-WL[k][f])- O[k+1][e][f]+ O[k][e][f] >=0 );

							   model.addConstr(50*TDL[k] + 6*(CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])+ O[k+1][e][f]- O[k][e][f] >=0 );
							   model.addConstr(50*TDL[k] + 6*(CR[k]) + 6*(1-WR[k][e])+ 6*(1-WR[k][f])- O[k+1][e][f]+ O[k][e][f] >=0 );
						   }
					   }
				   }
			   }

			   // if gene e(f) is in the left(right) interval, then its orientation gets turned => O_efk+1 != O_efk, (29)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   for(int f=0; f < genomSize; f++){
						   if(f != e){
							   model.addConstr(50*TDL[k] + 6*(1-WL[k][e]) +6*(1 -WR[k][f]) + O[k+1][e][f] -1 + O[k][e][f] >= 0 );
							   model.addConstr(50*TDL[k] + 6*(1-WL[k][e]) +6*(1 -WR[k][f]) - O[k+1][e][f] +1 - O[k][e][f] >= 0 );

							   model.addConstr(50*TDL[k] + 6*(1-WL[k][e]) +6*(1 -WR[k][f]) + O[k+1][f][e] -1 + O[k][f][e] >= 0 );
							   model.addConstr(50*TDL[k] + 6*(1-WL[k][e]) +6*(1 -WR[k][f]) - O[k+1][f][e] +1 - O[k][f][e] >= 0 );
						   }
					   }
				   }
			   }


			   // if gene e is not within left and right interval, then it stays to the left of all former left elements => O_efk+1 = O_efk, (28!)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   for(int f=0; f < genomSize; f++){
						   if(f != e){
							   model.addConstr(5*TDL[k] + WL[k][e] + WR[k][e] + O[k+1][e][f] - O[k][e][f] >= 0 );
							   model.addConstr(5*TDL[k] + WL[k][e] + WR[k][e] - O[k+1][e][f] + O[k][e][f] >= 0 );
						   }
					   }
				   }
			   }

			   // if gene e is not within left and right interval, then it stays to the right of all former right elements => O_efk+1 = O_efk, (29!)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   for(int f=0; f < genomSize; f++){
						   if(f != e){
							   model.addConstr(5*TDL[k] + WL[k][f] + WR[k][f] + O[k+1][e][f] - O[k][e][f] >= 0 );
							   model.addConstr(5*TDL[k] + WL[k][f] + WR[k][f] - O[k+1][e][f] + O[k][e][f] >= 0 );
						   }
					   }
				   }
			   }

			   // if element e is within an interval which gets inverted, then its sign gets changed => S_ek+1 != S_ek, (33)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + 1-CL[k] + 1-WL[k][e] + S[k+1][e] -1 + S[k][e] >= 0 );
					   model.addConstr(5*TDL[k] + 1-CL[k] + 1-WL[k][e] - S[k+1][e] +1 - S[k][e] >= 0 );

					   model.addConstr(5*TDL[k] + 1-CR[k] + 1-WR[k][e] + S[k+1][e] -1 + S[k][e] >= 0 );
					   model.addConstr(5*TDL[k] + 1-CR[k] + 1-WR[k][e] - S[k+1][e] +1 - S[k][e] >= 0 );
				   }
			   }

			   // if element e is not within an interval which gets inverted, then its sign stays the same => S_ek+1 = S_ek, (32!)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + CL[k] + 1-WL[k][e] + S[k+1][e] - S[k][e] >= 0 );
					   model.addConstr(5*TDL[k] + CL[k] + 1-WL[k][e] - S[k+1][e] + S[k][e] >= 0 );

					   model.addConstr(5*TDL[k] + CR[k] + 1-WR[k][e] + S[k+1][e] - S[k][e] >= 0 );
					   model.addConstr(5*TDL[k] + CR[k] + 1-WR[k][e] - S[k+1][e] + S[k][e] >= 0 );

				   }
			   }

			   // if element e is not within any interval, then its sign stays the same => S_ek+1 = S_ek, (33!)
			   for(int k=0; k < maxDis-1; k++){
				   for(int e=0; e < genomSize; e++){
					   model.addConstr(5*TDL[k] + WL[k][e] + WR[k][e] + S[k+1][e] - S[k][e] >= 0 );
					   model.addConstr(5*TDL[k] + WL[k][e] + WR[k][e] - S[k+1][e] + S[k][e] >= 0 );
				   }
			   }

			   // O_iik =0 for all i, k
			   for(int k=0; k< maxDis-1;k++){
				   for(int i=0; i< genomSize; i++){
					   model.addConstr(O[k+1][i][i] - O[k][i][i]<=0);
					   model.addConstr(O[k+1][i][i] - O[k][i][i]>=0);
				   }
			   }

			   //TODO
			   //forbids that TDRL simulate empty scenarios
//			   for(int k=0; k< maxDis-1; k++){
//				   GRBLinExpr temp_k = 0;
//				   GRBLinExpr temp_kk = 0;
//				   for(int q=0; q<genomSize-1;q++){
//					   for(int r=q+1; r < genomeSize; r++){
//						   temp_k += O[k][q][r];
//						   temp_kk += O[k+1][q][r];
//					   }
//				   }
//				   model.addConstr(100(1-TDL[k]) );
//			   }

			   // restrict events for the case of unequal costs
			   // no inversion in scenario
			   if(removeI){
				   for(int k=0; k<maxDis;k++){
					  model.addConstr(I[k]<=0);
				   }
			   }

			   // no trasposition in scenario
			   if(removeT){
				   for(int k=0; k<maxDis;k++){
					  model.addConstr(T[k]<=0);
				   }
			   }

			   // no inverse transposition in scenario
			   if(removeIT){
				   for(int k=0; k<maxDis;k++){
					  model.addConstr(U[k]<=0);
				   }
			   }

//////////////////////////////////////////////////////////////////// Optimize model /////////////////////////////////////////////////////
			   //until all solutions found or solutions objective is increased
			   alternative scen;
			   unsigned int numb_scen_sol=0;
			   do{
				   model.optimize();
				   #ifdef ILP_DEBUG
				   	   printmodel(model, maxDis, currentBest, upperBound, last_scenario_cost, numb_scen_sol, cutoff);
				   #endif //ILP_DEBUG
				   // if there was already an optimal solution and model is infeasible || cutoff || time_limit -> store set of scenarios
				   if((model.get(GRB_IntAttr_Status)==3 || model.get(GRB_IntAttr_Status)==6 || model.get(GRB_IntAttr_Status)==9) && !scen.isempty()){
					   alternative_scen.push_back(scen);
				   }

//////////////////////////////////////////////////////////////////// OUTPUT ////////////////////////////////////////////////////////////

				   objV=model.get(GRB_DoubleAttr_ObjVal);

				   //store solution of this ILP run, but only in the case that model is optimal solved, check status of optimization: 2== optimal, 9==time_limit
				   optimal = model.get(GRB_IntAttr_Status) == 2;
				   // find out if there is no solution due to cutoff constraint
				   bool foundFeasiblySolution = model.get(GRB_IntAttr_SolCount)>=1;
				   firstSol=model.get(GRB_IntAttr_SolCount) == 1;
				   time_exceeded=model.get(GRB_IntAttr_Status)==9;
				   if(time_exceeded){
					   break;
				   }
				   //cout << "status time limit" << model.get(GRB_IntAttr_Status)  << endl;
				   //cout << "solCount" << model.get(GRB_IntAttr_SolCount) << endl;
				   if(optimal || foundFeasiblySolution){//satisfy that model is solved, or one feasible solution found
					   objectValues->push_back(objV);
				   }

				   // prints the ILP problem as mps file in current directory
				   if(name != ""){
					   name.append(".mps");
					   model.write(name);
				   }

				   if(output && foundFeasiblySolution){
					   int* rowSum = new int[genomSize];
					   for(int k=0; k<maxDis-1; k++){
						   cout << "_________________" << "k=" << k << "__________________" << endl;
						   cout << "genome: " << endl;
						   // counts the sum of every row
						   for(int i=0; i< genomSize; i++){
							   rowSum[i]=0;
							   for(int j=0; j < genomSize; j++){
								   rowSum[i] = rowSum[i]+O[k][i][j].get(GRB_DoubleAttr_X);
							   }
						   }
						   // gives the right genorder out
						   for(int i=0; i< genomSize; i++){
							   for(int j=0; j <genomSize; j++){
								   if(rowSum[j]==genomSize-1-i){
									   if(S[k][j].get(GRB_DoubleAttr_X) == 1){
										   cout << "-"<< j << "\t";
										   //cout << (-1)*(j) << "\t";
									   }
									   else{
										   cout << j << "\t";
									   }
								   }
							   }
						   }
						   cout << endl;

						   cout << "O: " << endl;
						   for(int i=0; i<genomSize; i++){
							   for(int j=0; j< genomSize;j++){
								   if(j != genomSize-1){
									   cout << abs(O[k][i][j].get(GRB_DoubleAttr_X)) << " ";
								   }
								   else{
									   cout << abs(O[k][i][j].get(GRB_DoubleAttr_X)) << endl;
								   }
							   }
						   }

						   cout << "BL: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(BL[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "BM: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(BM[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "BR: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(BR[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "WL: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(WL[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "WR: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(WR[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "LM: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(LM[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "LR: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(LR[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "RL: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(RL[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "RM: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(RM[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "S: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(S[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "FS: " << endl;
						   for(int i=0; i<genomSize; i++){
							   cout << abs(FS[k][i].get(GRB_DoubleAttr_X)) << " ";
						   }
						   cout << endl;

						   cout << "I:" << "\t" << "T:" << "\t" <<  "iT:" << "\t" << "TDRL:" << endl;
						   cout << abs(I[k].get(GRB_DoubleAttr_X)) << "\t" << abs(T[k].get(GRB_DoubleAttr_X)) << "\t" <<  abs(U[k].get(GRB_DoubleAttr_X))
										<< "\t" <<  abs(TDL[k].get(GRB_DoubleAttr_X)) << endl;

						   cout << "CL:" << "\t" << "CR:" << endl;
						   cout << abs(CL[k].get(GRB_DoubleAttr_X)) << "\t" << abs(CR[k].get(GRB_DoubleAttr_X)) << endl;
					   }
					   delete[] rowSum;
				   }


////////////////////////////////// calculate the rrrmt scenario + store scenario and costs //////////////////////////////

				   //store costs of scenario including the score of children
				   //TODO
				   //	ms = objV;

				   if(useCrex2 && foundFeasiblySolution){
					   //removed additional costs of the children s.t. objV contains only cost of rrrmt by I,T,iT
					   for(int i=0; i<cdps.size();i++){
						   objV -= (1-S[0][abs(gpi[i])-1].get(GRB_DoubleAttr_X))*copy_cdps[i][INC] + S[0][abs(gpi[i])-1].get(GRB_DoubleAttr_X)*copy_cdps[i][DEC];
					   }
				   }

				   int* nrOnes;
				   nrOnes = new int[genomSize];
				   int* tempC;
				   tempC = new int[g1.size()];
				   int leftBound=0;
				   int middleBound=0;
				   int rightBound=0;

				   // only used for useCrex2 == true
				   // create scenarios and store ilp scenario and ilp scenario + child scenarios
				   unordered *comp_scen;			// ilp scenario+ child scenarios
				   comp_scen = new unordered;
				   ordered *scenario;				// ilp scenario
				   scenario = new ordered;

				   genom startG=g1;		// the starting genom of the ilp

				   if(useCrex2 && foundFeasiblySolution){
					   // calculate the starting genom of the ilp
					   for(int i=0; i<startG.size();i++){
						   startG[i]=0;
					   }
					   for(int m=0; m<startG.size(); m++){
						   tempC[m]=0;
					   }
					   // calculate how many ones are in every column of O[0] which implies the position of a gene
					   for(int col=0; col< startG.size(); col++){
						   for(int line=0; line< startG.size();line++){
							   tempC[col] += O[0][line][col].get(GRB_DoubleAttr_X);
						   }
					   }
					   //set elements of startG + adding signs
					   for(int i=0; i<startG.size();i++){
						   if(S[0][i].get(GRB_DoubleAttr_X)==1){
							   startG[tempC[i]]=-1*(i+1);
						   }else{
							   startG[tempC[i]]=i+1;
						   }
					   }
					   #ifdef ILP_DEBUG
						   //cout << "startG: " << startG << endl;
						   //cout << "startG an g2: " <<startG.identify_g(g2) << endl;
						   //cout << "startG an g1: " <<startG.identify_g(g1) << endl;
						   //cout << "startG an gpi: " <<startG.identify_g(gpi) << endl;
						   cout << "scenario calculated by ILP: " <<g2.identify_g(startG) << " to " << g2 << endl;
						   //cout << "g1 an startG: " <<g1.identify_g(startG) << endl;
						   //cout << "gpi an startG: " <<gpi.identify_g(startG) << endl;
					   #endif//ILP_DEBUG

					   //add rrrmt of childs to unordered right version
					   for(int i=0; i<cdpr.size();i++){
						   if(S[0][abs(gpi[i])-1].get(GRB_DoubleAttr_X) == 1){ // case negative sign
							   comp_scen->insert(copy_cdpr[i][DEC]->clone());
//							   #ifdef ILP_DEBUG
//								   cout << "case (-): add " << i << "-th child rrrmt: " << (*copy_cdpr[i][DEC]) << endl;
//								   cout << "not added: " << (*copy_cdpr[i][INC]) << endl;
//							   #endif//ILP_DEBUG
						   }else{									// case positive sign
							   comp_scen->insert(copy_cdpr[i][INC]->clone());
//							   #ifdef ILP_DEBUG
//								   cout << "case (+): add " << i << "-th child rrrmt: " << (*copy_cdpr[i][INC]) << endl;
//								   cout << "not added: " << (*copy_cdpr[i][DEC]) << endl;
//							   #endif//ILP_DEBUG
						   }
					   }
				   }
				   delete[] tempC;

//				   #ifdef ILP_DEBUG
//					   cout << "g1: " << g1 << endl;
//					   cout << "g2: " << g2 << endl;
//					   cout << "gpi: " << gpi << endl;
//					   cout << "cpg2: " << g1.identify_g(g2) << endl;
//				   #endif//ILP_DEBUG

				   //calculate rrrmt only in the case that a feasible solution was found
				   vector<rrrmt*> temp_rrrmt;
				   if(foundFeasiblySolution){
					   // create new version of g1, which is changed during the rrrmt creation in every run of ILP
					   genom temp_g1=g1;
					   //calculate rrrmts and add scenarios to ordered
					   for(int k=0; k<maxDis-1;k++){
						   // reset the settings for every k
						   leftBound=0;
						   middleBound=0;
						   rightBound=0;
						   for(int m=0; m<genomSize; m++){
							   nrOnes[m]=0;
						   }

						   // calculate how many ones are in every column of O[k]
						   for(int col=0; col< genomSize; col++){
							   for(int line=0; line< genomSize;line++){
								   nrOnes[col] += O[k][line][col].get(GRB_DoubleAttr_X);
							   }
						   }

						   // case inversion
						   if(I[k].get(GRB_DoubleAttr_X)==1){
							   //	build inversion(rev) rrrmt
							   for(int i=0; i<genomSize; i++){
								   if(BL[k][i].get(GRB_DoubleAttr_X)==1){
									   leftBound=nrOnes[i];
								   }
								   if(BR[k][i].get(GRB_DoubleAttr_X)==1){
									   rightBound=nrOnes[i]-1;
								   }
							   }
							   middleBound =-1;
							   //define inversion
							   rrrmt *r;
							   r = new rev( leftBound, rightBound, temp_g1 ); //cpg1
//							   #ifdef ILP_DEBUG
//								   cout << "inversion: " << (*r) << endl;
//							   #endif//ILP_DEBUG
							   // this is done since a following rrrmt is applied to a result of this operation to former g1
							   r->apply(temp_g1);
							   //cout << leftBound << " " << rightBound << endl;
							   // add inversion to the scenario
							   if(useCrex2){
								   //push_back(r);
								   // up to now alternative rearrangement scenarios for the quotient
								   // permutation have been constructed. by renaming the elements that are
								   // affected by the rearrangements with the element sets corresponding
								   // to the child nodes of the pnode scenarios for the permutations
								   // are gained
								   r->rename(mapping, origin.get_nmap(), origin.size());
//								   #ifdef ILP_DEBUG
//									   cout << "inversion: " << (*r) << endl;
//								   #endif//ILP_DEBUG
								   scenario->push_back(r->clone());
							   }else{
								   if(all==1){
									   this->push_back(r);
									   temp_rrrmt.push_back(r);
								   }else{
									   temp_rrrmt.push_back(r);
								   }
							   }
							   //delete r; doesn't seem to work here (including cloning in 2144 and 2146)
						   }

						   // case transposition
						   if(T[k].get(GRB_DoubleAttr_X)==1){
							   //	build transposition(transp) rrrmt
							   for(int i=0; i<genomSize; i++){
								   if(BL[k][i].get(GRB_DoubleAttr_X)==1){
									   leftBound=nrOnes[i];
								   }
								   if(BM[k][i].get(GRB_DoubleAttr_X)==1){
									   middleBound=nrOnes[i];
								   }
								   if(BR[k][i].get(GRB_DoubleAttr_X)==1){
									   rightBound=nrOnes[i];
								   }
							   }
							   //define transposition
							   rrrmt *t;
							   t = new transp( leftBound, middleBound, rightBound, temp_g1 );
//							   #ifdef ILP_DEBUG
//								   cout << "transposition: " << (*t) << endl;
//							   #endif//ILP_DEBUG
							   // this is done since a following rrrmt is applied to a result of this operation to former g1
							   t->apply(temp_g1);
							   //cout << leftBound << " " << middleBound << " " << rightBound << endl;

							   // add transposition to the scenario
							   if(useCrex2){
								   //push_back(t);
								   // up to now alternative rearrangement scenarios for the quotient
								   // permutation have been constructed. by renaming the elements that are
								   // affected by the rearrangements with the element sets corresponding
								   // to the child nodes of the pnode scenarios for the permutations
								   // are gained
								   t->rename(mapping, origin.get_nmap(), origin.size());
								   scenario->push_back(t->clone());
							   }else{
								   if(all==1){
									   this->push_back(t);
									   temp_rrrmt.push_back(t);
								   }else{
									   temp_rrrmt.push_back(t);
								   }
							   }
							   //delete t;
						   }

						   // case inverse transposition
						   if(U[k].get(GRB_DoubleAttr_X)==1){
							   //	build inverse transposition(revtransp) rrrmt
							   if(CL[k].get(GRB_DoubleAttr_X)==1){
								   for(int i=0; i<genomSize; i++){
									   if(BL[k][i].get(GRB_DoubleAttr_X)==1){
										   leftBound=nrOnes[i];
									   }
									   if(BM[k][i].get(GRB_DoubleAttr_X)==1){
										   middleBound=nrOnes[i]-1;
									   }
									   if(BR[k][i].get(GRB_DoubleAttr_X)==1){
										   rightBound=nrOnes[i];
									   }
								   }

								   //define inverse transposition
								   rrrmt *it;

								   // left sequence gets inverted
								   it = new revtransp( leftBound, middleBound, rightBound, temp_g1 );
//								   #ifdef ILP_DEBUG
//									   cout << "inverse transposition: " << (*it) << endl;
//								   #endif//ILP_DEBUG
								   // this is done since a following rrrmt is applied to a result of this operation to former g1
								   it->apply(temp_g1);
								   //cout << leftBound << " " << middleBound << " " << rightBound << endl;
								   // add inverse transposition to the scenario
								   if(useCrex2){
									   //push_back(it);
									   // up to now alternative rearrangement scenarios for the quotient
									   // permutation have been constructed. by renaming the elements that are
									   // affected by the rearrangements with the element sets corresponding
									   // to the child nodes of the pnode scenarios for the permutations
									   // are gained
									   it->rename(mapping, origin.get_nmap(), origin.size());
									   scenario->push_back(it->clone());
								   }else{
									   if(all==1){
										   this->push_back(it);
										   temp_rrrmt.push_back(it);
									   }else{
										   temp_rrrmt.push_back(it);
									   }
								   }
							   }
							   if(CR[k].get(GRB_DoubleAttr_X)==1){
								   for(int i=0; i<genomSize; i++){
									   if(BL[k][i].get(GRB_DoubleAttr_X)==1){
										   leftBound=nrOnes[i];
									   }
									   if(BM[k][i].get(GRB_DoubleAttr_X)==1){
										   middleBound=nrOnes[i];
									   }
									   if(BR[k][i].get(GRB_DoubleAttr_X)==1){
										   rightBound=nrOnes[i]-1;
									   }
								   }
								   //define inverse transposition
								   rrrmt *it;
								   // right sequence gets inverted
								   it = new revtransp( middleBound, rightBound, leftBound, temp_g1 );
//								   #ifdef ILP_DEBUG
//									   cout << "inverse transposition: " << (*it) << endl;
//								   #endif//ILP_DEBUG
								   // this is done since a following rrrmt is applied to a result of this operation to former g1
								   it->apply(temp_g1);
								   //cout << middleBound << " " << rightBound << " " << leftBound << endl;
								   // add inverse transposition to the scenario
								   if(useCrex2){
									   //push_back(it);
									   // up to now alternative rearrangement scenarios for the quotient
									   // permutation have been constructed. by renaming the elements that are
									   // affected by the rearrangements with the element sets corresponding
									   // to the child nodes of the pnode scenarios for the permutations
									   // are gained
									   it->rename(mapping, origin.get_nmap(), origin.size());
									   scenario->push_back(it->clone());
								   }else{
									   if(all==1){
										   this->push_back(it);
										   temp_rrrmt.push_back(it);
									   }else{
										   temp_rrrmt.push_back(it);
									   }
								   }
							   }
						   }

						   //case TDRL //TODO
						   if(TDL[k].get(GRB_DoubleAttr_X)==1){
							   //	build tdrl rrrmt
							   vector<bool> event(genomSize,0);
							   for( int i=0; i<genomSize;i++){
								   if(FS[k][i].get(GRB_DoubleAttr_X)==1){
									   event[ nrOnes[i] ]=false;
								   }else{
									   event[ nrOnes[i] ]=true;
								   }
							   }
							   //define tdrl
							   rrrmt *it;
							   // left sequence gets inverted
							   it = new tdrl(event, temp_g1);
//							   #ifdef ILP_DEBUG
//								   cout << "tdrl: " << (*it) << endl;
//							   #endif//ILP_DEBUG
							   // this is done since a following rrrmt is applied to a result of this operation to former g1
							   it->apply(temp_g1);
							   // add tdrl to the scenario
							   if(useCrex2){
								   // push_back(it);
								   // up to now alternative rearrangement scenarios for the quotient
								   // permutation have been constructed. by renaming the elements that are
								   // affected by the rearrangements with the element sets corresponding
								   // to the child nodes of the pnode scenarios for the permutations
								   // are gained
								   it->rename(mapping, origin.get_nmap(), origin.size());
								   scenario->push_back(it->clone());
							   }else{
								   if(all==1){
									   this->push_back(it);
									   temp_rrrmt.push_back(it);
								   }else{
									   temp_rrrmt.push_back(it);
								   }
							   }
						   }
					   }

					   if(useCrex2){
						   // insert ilp scenario
						   comp_scen->insert(scenario->clone());
						   // simplify scenario and return complete scenario (ilp + child_scenarios)
						   // mr=(comp_scen->clone())->simplify();
						   objectUnordered->push_back(comp_scen->clone());
						   delete comp_scen;
						   objectOrdered->push_back(scenario->clone());
					   }
				   }
				   #ifdef ILP_ALL_DEBUG
					   if(foundFeasiblySolution){
						   cout << "Objective Value: " << objV << endl;
						   cout << "reconstructed scenario: " << endl;
						   for(int i=0; i<temp_rrrmt.size();i++){
							   cout << i << "\t" << *(temp_rrrmt[i]) << endl;
						   }
					   }
				   #endif //ILP_ALL_DEBUG

				   if(foundFeasiblySolution){
					   if(!objectValues->empty() && currentBest < model.get(GRB_DoubleParam_Cutoff)){
						   model.addConstr(obj-objectValues->back()<=0);
					   }
					   if(objV<=last_scenario_cost && numb_scen_sol < all ){
						   numb_scen_sol++;
						   last_scenario_cost=objV;
						   ordered *temp_ordered;
						   temp_ordered= new ordered(temp_rrrmt,true);
						   scen.insert(temp_ordered);
						   //add new constraint
						   GRBLinExpr constraint = 0;
						   for(int k=0; k<maxDis-1;k++){
							   if(I[k].get(GRB_DoubleAttr_X)==1){
								   constraint += 1-I[k];
							   }
							   if(T[k].get(GRB_DoubleAttr_X)==1){
								   constraint += 1-T[k];
							   }
							   if(U[k].get(GRB_DoubleAttr_X)==1){
								   constraint += 1-U[k];
							   }
							   if(CL[k].get(GRB_DoubleAttr_X)==1){
								   constraint += 1-CL[k];
							   }
							   if(CR[k].get(GRB_DoubleAttr_X)==1){
								   constraint += 1-CR[k];
							   }
							   //breakpoints
							   for(int i=0; i< genomSize; i++){
								   if(BL[k][i].get(GRB_DoubleAttr_X)==1){
									   constraint += 1-BL[k][i];
								   }
								   if(BM[k][i].get(GRB_DoubleAttr_X)==1){
									   constraint += 1-BM[k][i];
								   }
								   if(BR[k][i].get(GRB_DoubleAttr_X)==1){
									   constraint += 1-BR[k][i];
								   }
							   }
						   }
						   model.addConstr(constraint -1 >=0);
						   model.update();

						   if(numb_scen_sol >= all){
							   alternative_scen.push_back(scen);
						   }
					   }else{
						   alternative_scen.push_back(scen);
					   }
				   }

				   //set cutoff with up to now best object value
				   if(!objectValues->empty() && currentBest > objectValues->back()){
					   currentBest=objectValues->back();
				   }
				   delete[] nrOnes;
				   if(!useCrex2){
					   delete comp_scen;			// ilp scenario+ child scenarios
				   }
				   delete scenario;

			   }while(objV<=last_scenario_cost && numb_scen_sol < all);//do while loop for all>=1

			   //delete variables
			   delete[] I;
			   delete[] T;
			   delete[] U;
			   delete[] TDL;
			   delete[] CL;
			   delete[] CR;
			   for(int h=0; h<maxDis;h++){
				   delete[] LR[h];
				   delete[] LM[h];
				   delete[] RL[h];
				   delete[] RM[h];
				   delete[] BL[h];
				   delete[] BM[h];
				   delete[] BR[h];
				   delete[] WL[h];
				   delete[] WR[h];
				   delete[] S[h];
				   delete[] FS[h];
				   for(int g=0; g<genomSize;g++){
					   delete[] O[h][g];
				   }
				   delete[] O[h];
			   }
			   delete[] O;
			   delete[] LR;
			   delete[] LM;
			   delete[] RL;
			   delete[] RM;
			   delete[] BL;
			   delete[] BM;
			   delete[] BR;
			   delete[] WL;
			   delete[] WR;
			   delete[] S;
			   delete[] FS;

			}catch(GRBException e) {
				// ilp is iteration based, which means that ilp problem might be infeasible
				// in this case it is called which larger maxDis and an exception with code 10005 is thown,
				// because some gurobi variables are not assigned
				// there is no need to read this error message
				if(e.getErrorCode()!=10005 && e.getErrorCode()!=10004){
					cerr << "Error code = " << e.getErrorCode() << endl;
					cerr << e.getMessage() << endl;
				}
			}catch(...) {
				cerr << "Exception during optimization" << endl;
				exit(EXIT_FAILURE);
				break; /*should not be reached*/
			}

			if(use_cutoff && objectValues->size()==0){
				upperBound = ceil(cutoff/minimumCosts);
				//upperBound=cutoff + ceil((max_cost-min_cost)/minimumCosts);
			}

			// calculate upper bound for the number of checked rrrmts*
			if(firstSol && optimal){
				if(useCrex2){
					upperBound=objV + ceil((max_cost-min_cost)/minimumCosts);
				}else{
					if(noAllEvents){
						//cheapest scenario is done only with cheapest rrrmt
						upperBound= objV/minimumCosts;
					}else{
						//end do-while loop since the parsimonious event is already found
						upperBound=0;
					}
				}
			}
			// stop calculation, since the scenario shall only be constructed for a specific number of rrrmts
			if(fixed_numb_rrrmt){
				alternative temp_scen;
				alternative_scen.push_back(temp_scen);
				break;
			}
			if((maxDis-1)*minimumCosts>=cutoff && use_cutoff && objV==0){
				// cutoff is to small --> so solution can be found
				is_cutoff=true;
				alternative temp_scen;
				alternative_scen.push_back(temp_scen);
				break;
			}
		}while(maxDis<upperBound || objV==0);	// objV==0 is for the case that the ILP is infeasible in first run
											// maxDis<upperBound satisfies that ILP checks all possible distances
		//calculate parsimonious scenario and store costs and scenario
		//get index of parsimonious scenario
		int index=std::numeric_limits<int>::max();
		float tempValue=std::numeric_limits<int>::max();
		for(int i=0; i<objectValues->size();i++){
			if(objectValues->at(i)<tempValue){
				index=i;
				tempValue=objectValues->at(i);
			}
		}

		//store costs
		if(!objectValues->empty()){
			ms=objectValues->at(index);
		}

		if(objectValues->empty() && !time_exceeded){
			is_cutoff=true;
		}

		//store scenario
		if(!objectUnordered->empty()){
			mr=objectUnordered->at(index)->clone();
			mr->simplify();
		}

		//store scenario of ILP without child scenarios
		if(!objectOrdered->empty()){
			push_back(objectOrdered->at(index)->clone());
			this->simplify();
		}

		delete i;
		delete iT;
		delete T;
		delete tdl;
		for(int h=0; h<genomSize;h++){
			delete[] genomStart[h];
		}
		delete[] genomStart;
		delete objectOrdered;
		delete objectUnordered;
		delete objectValues;
	}
}

//////////////////////////////////////////////////////////////////////////////

ilp_rrrmt::ilp_rrrmt(const genom &g1, const costfoo *cst, float &objV){

	/////////////// pre ILP stuff //////////////////////////////////////////////////////////////////////////////////
	unsigned dist=10;
	unsigned genome_size=g1.size();

	// TODO Calculate P[i] the distance of element |i| from its actual position in g1

	///////////////	add ILP //////////////////////////////////////////////////////////////////////////////////////////////
	try {
		// create gurobi environement
		GRBEnv env = GRBEnv();
		// produce output ??
		//if(!output){
			env.set(GRB_IntParam_OutputFlag,0);
		//}
		// only use specific number of threads
		//env.set(GRB_IntParam_Threads, 1);

		// create gurobi model
		GRBModel model = GRBModel(env);

		//////////////////////// define variables //////////////////////////////////////////////////////////////////
		// S_ik contains the sign of a gene, S_ik = 1 <-> gene i has a negative sign
		GRBVar** I;
		I = new GRBVar*[dist];
		for( int k=0; k< dist; k++){
			I[k]= new GRBVar[genome_size];
		}

		for(int k=0; k<dist; k++){
			for(int i=0; i<genome_size; i++){
				I[k][i] = model.addVar((-1)*(genome_size-1), genome_size-1, 0.0, GRB_INTEGER);
			}
		}

		///////////////////////// define constraints ///////////////////////////////////////////////////////////////
//		for(int k=0; k < maxDis; k++){
//		   for(int i=0; i < genomSize; i++){
//			   for(int j=0; j < genomSize; j++){
//					   model.addConstr(50*(1-U[k])+ 5*(1-BM[k][i])+ 5*(1-BR[k][j]) + O[k][i][j] -1 >=0);
//			   }
//		   }
//		}


	}catch(GRBException e) {
					// ilp is iteration based, which means that ilp problem might be infeasible
					// in this case it is called which larger maxDis and an exception with code 10005 is thrown,
					// because some gurobi variables are not assigned
					// there is no need to read this error message
					if(e.getErrorCode()!=10005 && e.getErrorCode()!=10004){
						cerr << "Error code = " << e.getErrorCode() << endl;
						cerr << e.getMessage() << endl;
					}
	}catch(...) {
		cerr << "Exception during optimization" << endl;
		exit(EXIT_FAILURE);
	}



}

//////////////////////////////////////////////////////////////////////////////

string ilp_rrrmt::typestr() const{
	return "ilp_rrrmt";
}

//////////////////////////////////////////////////////////////////////////////

void printmodel(const GRBModel &model, int maxDis, float currentBest, int upperBound, double last_scenario_cost, unsigned int numb_scen_sol, float cutoff){
	cout << "**" <<model.get(GRB_IntAttr_Status) << "\t" <<  maxDis-1 << "\t" << currentBest << "\t" << upperBound
			<< "\t" << last_scenario_cost << "\t" << numb_scen_sol << "\t" << model.get(GRB_DoubleParam_TimeLimit)
			<< "\t" << cutoff << "\t" << model.get(GRB_IntAttr_SolCount);
	if(model.get(GRB_IntAttr_SolCount) != 0){
		cout << "\t" << model.get(GRB_DoubleAttr_ObjVal) << endl;
	}else{
		cout << "\t" << "--" << endl;
	}
//	cout << "status of model: " << model.get(GRB_IntAttr_Status);
//	if(model.get(GRB_IntAttr_Status)==1){
//		cout << " (model is loaded, but no solution information is available)" << endl;
//	}else if(model.get(GRB_IntAttr_Status)==2){
//		cout << " (model is solved optimally)" << endl;
//	}else if(model.get(GRB_IntAttr_Status)==3){
//		cout << " (model is infeasible)" << endl;
//	}else if(model.get(GRB_IntAttr_Status)==6){
//		cout << " (solution is larger than cutoff)" << endl;
//	}else if(model.get(GRB_IntAttr_Status)==9){
//		cout << " (time limit exceeded)" << endl;
//	}else{
//		cerr << "unknown model status: " << model.get(GRB_IntAttr_Status) << " --> exiting!" << endl;
//		exit(EXIT_FAILURE);
//	}
//	cout << "# of rrrmts: " << maxDis-1 << endl;
//	cout << "current best objective: " << currentBest << endl;
//	cout << "upper bound on the number of rrrmt: " << upperBound << endl;
//	cout << "cost of the last scenario: " << last_scenario_cost << endl;
//	cout << "number of solution scenarios: " << numb_scen_sol << endl;
//	if(model.get(GRB_IntAttr_SolCount) != 0){
//		cout << "objective found: " << model.get(GRB_DoubleAttr_ObjVal) << endl;
//	}else{
//		cout << "time_limit: " << model.get(GRB_DoubleParam_TimeLimit) << endl;
//		cout << "cutoff: " << cutoff << endl;
//		cout << "number of solution found: " << model.get(GRB_IntAttr_SolCount) << endl;
//	}
}

